/*******************************************
 *
 * $GAMGI/src/chem/gamgi_chem_atom.c
 *
 * Copyright (C) 2001, 2004 Carlos Pereira
 *
 * Distributed under the terms of the GNU
 * General Public License: $GAMGI/LICENSE
 *
 */

#include "gamgi_engine.h"
#include "gamgi_chem.h"
#include "gamgi_global.h"

#include "gamgi_engine_remove.h"
#include "gamgi_engine_unlink.h"
#include "gamgi_engine_reparent.h"
#include "gamgi_math_hash.h"
#include "gamgi_phys_plane.h"
#include "gamgi_phys_direction.h"
#include "gamgi_expat_export.h"

/************** external function **************
 *                                             *
 *             GAMGI_CHEM_ATOM_NAME            *
 *                                             *
 * Get the element name from the atomic number *
 ***********************************************/

void gamgi_chem_atom_name (int number, char *name)
{
static char *element[] = {"Du", 
"H", "He",

"Li", "Be", "B", "C", "N", "O", "F", "Ne",

"Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar",

"K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co",
"Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", "Kr",

"Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh",
"Pd", "Ag", "Cd", "In", "Sn", "Sb", "Te", "I", "Xe",

"Cs", "Ba",

"La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", 
"Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu",

"Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", 
"Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn",

"Fr", "Ra",

"Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm",
"Bk", "Cf", "Es", "Fm", "Md", "No", "Lr",

"Rf", "Db", "Sg", "Bh", "Hs", "Mt", "Ds", "Rg"};

/**************************************
 * For speed reasons, this function   *
 * does NOT check if number is valid: *
 * 0 <= number <= GAMGI_CHEM_ATOM_MAX *
 *   In the few cases where this is   *
 *   unsafe, number MUST be checked   *
 *   BEFORE calling this function!    *
 **************************************/

strcpy (name, element[number]);
}

/************* external function ***************
 *                                             *
 *       GAMGI_CHEM_CREATE_ATOM_NUMBER         *
 *                                             *
 * Get the atomic number from the element name *
 ***********************************************/

gamgi_bool gamgi_chem_atom_number (const char *name, int *number)
{
char element[4];
gamgi_slist *slist;
unsigned int hash;

hash = gamgi_math_hash_value (name, GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);

for (slist = gamgi->element[hash]; slist != NULL; slist = slist->next)
  {
  *number = GAMGI_POINTER_TO_INT (slist->data);
  gamgi_chem_atom_name (*number, element);
  if (strcmp (name, element) == 0) return TRUE;
  }

return FALSE;
}

gamgi_slist **gamgi_chem_atom_element (void)
{
gamgi_slist **table;
unsigned int hash;

/*********************************
 * create hash table to speed up *
 * atom element name recognition *
 *********************************/

table = gamgi_math_hash_create (GAMGI_CHEM_HASH_SIZE);

hash = gamgi_math_hash_value ("Du", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_DU);

hash = gamgi_math_hash_value ("H", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_H);

hash = gamgi_math_hash_value ("He", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_HE);

hash = gamgi_math_hash_value ("Li", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_LI);

hash = gamgi_math_hash_value ("Be", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_BE);

hash = gamgi_math_hash_value ("B", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_B);

hash = gamgi_math_hash_value ("C", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_C);

hash = gamgi_math_hash_value ("N", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_N);

hash = gamgi_math_hash_value ("O", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_O);

hash = gamgi_math_hash_value ("F", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_F);

hash = gamgi_math_hash_value ("Ne", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_NE);

hash = gamgi_math_hash_value ("Na", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_NA);

hash = gamgi_math_hash_value ("Mg", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_MG);

hash = gamgi_math_hash_value ("Al", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AL);

hash = gamgi_math_hash_value ("Si", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SI);

hash = gamgi_math_hash_value ("P", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_P);

hash = gamgi_math_hash_value ("S", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_S);

hash = gamgi_math_hash_value ("Cl", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CL);

hash = gamgi_math_hash_value ("Ar", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AR);

hash = gamgi_math_hash_value ("K", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_K);

hash = gamgi_math_hash_value ("Ca", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CA);

hash = gamgi_math_hash_value ("Sc", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SC);

hash = gamgi_math_hash_value ("Ti", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TI);

hash = gamgi_math_hash_value ("V", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_V);

hash = gamgi_math_hash_value ("Cr", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CR);

hash = gamgi_math_hash_value ("Mn", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_MN);

hash = gamgi_math_hash_value ("Fe", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_FE);

hash = gamgi_math_hash_value ("Co", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CO);

hash = gamgi_math_hash_value ("Ni", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_NI);

hash = gamgi_math_hash_value ("Cu", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CU);

hash = gamgi_math_hash_value ("Zn", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_ZN);

hash = gamgi_math_hash_value ("Ga", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_GA);

hash = gamgi_math_hash_value ("Ge", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_GE);

hash = gamgi_math_hash_value ("As", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AS);

hash = gamgi_math_hash_value ("Se", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SE);

hash = gamgi_math_hash_value ("Br", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_BR);

hash = gamgi_math_hash_value ("Kr", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_KR);

hash = gamgi_math_hash_value ("Rb", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RB);

hash = gamgi_math_hash_value ("Sr", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SR);

hash = gamgi_math_hash_value ("Y", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_Y);

hash = gamgi_math_hash_value ("Zr", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_ZR);

hash = gamgi_math_hash_value ("Nb", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_NB);

hash = gamgi_math_hash_value ("Mo", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_MO);

hash = gamgi_math_hash_value ("Tc", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TC);

hash = gamgi_math_hash_value ("Ru", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RU);

hash = gamgi_math_hash_value ("Rh", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RH);

hash = gamgi_math_hash_value ("Pd", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PD);

hash = gamgi_math_hash_value ("Ag", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AG);

hash = gamgi_math_hash_value ("Cd", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CD);

hash = gamgi_math_hash_value ("In", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_IN);

hash = gamgi_math_hash_value ("Sn", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SN);

hash = gamgi_math_hash_value ("Sb", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SB);

hash = gamgi_math_hash_value ("Te", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TE);

hash = gamgi_math_hash_value ("I", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_I);

hash = gamgi_math_hash_value ("Xe", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_XE);

hash = gamgi_math_hash_value ("Cs", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CS);

hash = gamgi_math_hash_value ("Ba", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_BA);

hash = gamgi_math_hash_value ("La", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_LA);

hash = gamgi_math_hash_value ("Ce", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CE);

hash = gamgi_math_hash_value ("Pr", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PR);

hash = gamgi_math_hash_value ("Nd", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_ND);

hash = gamgi_math_hash_value ("Pm", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PM);

hash = gamgi_math_hash_value ("Sm", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SM);

hash = gamgi_math_hash_value ("Eu", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_EU);

hash = gamgi_math_hash_value ("Gd", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_GD);

hash = gamgi_math_hash_value ("Tb", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TB);

hash = gamgi_math_hash_value ("Dy", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_DY);

hash = gamgi_math_hash_value ("Ho", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_HO);

hash = gamgi_math_hash_value ("Er", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_ER);

hash = gamgi_math_hash_value ("Tm", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TM);

hash = gamgi_math_hash_value ("Yb", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_YB);

hash = gamgi_math_hash_value ("Lu", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_LU);

hash = gamgi_math_hash_value ("Hf", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_HF);

hash = gamgi_math_hash_value ("Ta", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TA);

hash = gamgi_math_hash_value ("W", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_W);

hash = gamgi_math_hash_value ("Re", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RE);

hash = gamgi_math_hash_value ("Os", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_OS);

hash = gamgi_math_hash_value ("Ir", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_IR);

hash = gamgi_math_hash_value ("Pt", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PT);

hash = gamgi_math_hash_value ("Au", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AU);

hash = gamgi_math_hash_value ("Hg", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_HG);

hash = gamgi_math_hash_value ("Tl", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TL);

hash = gamgi_math_hash_value ("Pb", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PB);

hash = gamgi_math_hash_value ("Bi", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_BI);

hash = gamgi_math_hash_value ("Po", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PO);

hash = gamgi_math_hash_value ("At", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AT);

hash = gamgi_math_hash_value ("Rn", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RN);

hash = gamgi_math_hash_value ("Fr", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_FR);

hash = gamgi_math_hash_value ("Ra", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RA);

hash = gamgi_math_hash_value ("Ac", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AC);

hash = gamgi_math_hash_value ("Th", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_TH);

hash = gamgi_math_hash_value ("Pa", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PA);

hash = gamgi_math_hash_value ("U", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_U);

hash = gamgi_math_hash_value ("Np", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_NP);

hash = gamgi_math_hash_value ("Pu", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_PU);

hash = gamgi_math_hash_value ("Am", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_AM);

hash = gamgi_math_hash_value ("Cm", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CM);

hash = gamgi_math_hash_value ("Bk", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_BK);

hash = gamgi_math_hash_value ("Cf", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_CF);

hash = gamgi_math_hash_value ("Es", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_ES);

hash = gamgi_math_hash_value ("Fm", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_FM);

hash = gamgi_math_hash_value ("Md", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_MD);

hash = gamgi_math_hash_value ("No", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_NO);

hash = gamgi_math_hash_value ("Lr", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_LR);

hash = gamgi_math_hash_value ("Rf", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RF);

hash = gamgi_math_hash_value ("Db", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_DB);

hash = gamgi_math_hash_value ("Sg", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_SG);

hash = gamgi_math_hash_value ("Bh", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_BH);

hash = gamgi_math_hash_value ("Hs", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_HS);

hash = gamgi_math_hash_value ("Mt", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_MT);

hash = gamgi_math_hash_value ("Ds", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_DS);

hash = gamgi_math_hash_value ("Rg", GAMGI_CHEM_HASH_MULTIPLIER, GAMGI_CHEM_HASH_SIZE);
table[hash] = gamgi_math_hash_add (table[hash], (void *) GAMGI_CHEM_RG);

return table;
}

/********* external function *********
 *                                   *
 *       GAMGI_CHEM_ATOM_MASS        *
 *                                   *
 * Set default mass for each element *
 *************************************/

void gamgi_chem_atom_mass (gamgi_atom_class *atom_class)
{
double *mass = atom_class->mass;

mass[GAMGI_CHEM_DU] = GAMGI_CHEM_DU_MASS;

mass[GAMGI_CHEM_H] = GAMGI_CHEM_H_MASS;

mass[GAMGI_CHEM_HE] = GAMGI_CHEM_HE_MASS;

mass[GAMGI_CHEM_LI] = GAMGI_CHEM_LI_MASS;

mass[GAMGI_CHEM_BE] = GAMGI_CHEM_BE_MASS;

mass[GAMGI_CHEM_B] = GAMGI_CHEM_B_MASS;

mass[GAMGI_CHEM_C] = GAMGI_CHEM_C_MASS;

mass[GAMGI_CHEM_N] = GAMGI_CHEM_N_MASS;

mass[GAMGI_CHEM_O] = GAMGI_CHEM_O_MASS;

mass[GAMGI_CHEM_F] = GAMGI_CHEM_F_MASS;

mass[GAMGI_CHEM_NE] = GAMGI_CHEM_NE_MASS;

mass[GAMGI_CHEM_NA] = GAMGI_CHEM_NA_MASS;

mass[GAMGI_CHEM_MG] = GAMGI_CHEM_MG_MASS;

mass[GAMGI_CHEM_AL] = GAMGI_CHEM_AL_MASS;

mass[GAMGI_CHEM_SI] = GAMGI_CHEM_SI_MASS;
  
mass[GAMGI_CHEM_P] = GAMGI_CHEM_P_MASS;

mass[GAMGI_CHEM_S] = GAMGI_CHEM_S_MASS;

mass[GAMGI_CHEM_CL] = GAMGI_CHEM_CL_MASS;

mass[GAMGI_CHEM_AR] = GAMGI_CHEM_AR_MASS;

mass[GAMGI_CHEM_K] = GAMGI_CHEM_K_MASS;

mass[GAMGI_CHEM_CA] = GAMGI_CHEM_CA_MASS;
  
mass[GAMGI_CHEM_SC] = GAMGI_CHEM_SC_MASS;

mass[GAMGI_CHEM_TI] = GAMGI_CHEM_TI_MASS;

mass[GAMGI_CHEM_V] = GAMGI_CHEM_V_MASS;

mass[GAMGI_CHEM_CR] = GAMGI_CHEM_CR_MASS;

mass[GAMGI_CHEM_MN] = GAMGI_CHEM_MN_MASS;

mass[GAMGI_CHEM_FE] = GAMGI_CHEM_FE_MASS;
  
mass[GAMGI_CHEM_CO] = GAMGI_CHEM_CO_MASS;

mass[GAMGI_CHEM_NI] = GAMGI_CHEM_NI_MASS;

mass[GAMGI_CHEM_CU] = GAMGI_CHEM_CU_MASS;

mass[GAMGI_CHEM_ZN] = GAMGI_CHEM_ZN_MASS;

mass[GAMGI_CHEM_GA] = GAMGI_CHEM_GA_MASS;

mass[GAMGI_CHEM_GE] = GAMGI_CHEM_GE_MASS;
  
mass[GAMGI_CHEM_AS] = GAMGI_CHEM_AS_MASS;

mass[GAMGI_CHEM_SE] = GAMGI_CHEM_SE_MASS;

 mass[GAMGI_CHEM_BR] = GAMGI_CHEM_BR_MASS;

mass[GAMGI_CHEM_KR] = GAMGI_CHEM_KR_MASS;

mass[GAMGI_CHEM_RB] = GAMGI_CHEM_RB_MASS;

mass[GAMGI_CHEM_SR] = GAMGI_CHEM_SR_MASS;
  
mass[GAMGI_CHEM_Y] = GAMGI_CHEM_Y_MASS;

mass[GAMGI_CHEM_ZR] = GAMGI_CHEM_ZR_MASS;

mass[GAMGI_CHEM_NB] = GAMGI_CHEM_NB_MASS;

mass[GAMGI_CHEM_MO] = GAMGI_CHEM_MO_MASS;

mass[GAMGI_CHEM_TC] = GAMGI_CHEM_TC_MASS;

mass[GAMGI_CHEM_RU] = GAMGI_CHEM_RU_MASS;
  
mass[GAMGI_CHEM_RH] = GAMGI_CHEM_RH_MASS;

mass[GAMGI_CHEM_PD] = GAMGI_CHEM_PD_MASS;

mass[GAMGI_CHEM_AG] = GAMGI_CHEM_AG_MASS;

mass[GAMGI_CHEM_CD] = GAMGI_CHEM_CD_MASS;

mass[GAMGI_CHEM_IN] = GAMGI_CHEM_IN_MASS;

mass[GAMGI_CHEM_SN] = GAMGI_CHEM_SN_MASS;
  
mass[GAMGI_CHEM_SB] = GAMGI_CHEM_SB_MASS;

mass[GAMGI_CHEM_TE] = GAMGI_CHEM_TE_MASS;

mass[GAMGI_CHEM_I] = GAMGI_CHEM_I_MASS;

mass[GAMGI_CHEM_XE] = GAMGI_CHEM_XE_MASS;

mass[GAMGI_CHEM_CS] = GAMGI_CHEM_CS_MASS;

mass[GAMGI_CHEM_BA] = GAMGI_CHEM_BA_MASS;

mass[GAMGI_CHEM_LA] = GAMGI_CHEM_LA_MASS;

mass[GAMGI_CHEM_CE] = GAMGI_CHEM_CE_MASS;

mass[GAMGI_CHEM_PR] = GAMGI_CHEM_PR_MASS;

mass[GAMGI_CHEM_ND] = GAMGI_CHEM_ND_MASS;

mass[GAMGI_CHEM_PM] = GAMGI_CHEM_PM_MASS;

mass[GAMGI_CHEM_SM] = GAMGI_CHEM_SM_MASS;

mass[GAMGI_CHEM_EU] = GAMGI_CHEM_EU_MASS;

mass[GAMGI_CHEM_GD] = GAMGI_CHEM_GD_MASS;

mass[GAMGI_CHEM_TB] = GAMGI_CHEM_TB_MASS;

mass[GAMGI_CHEM_DY] = GAMGI_CHEM_DY_MASS;

mass[GAMGI_CHEM_HO] = GAMGI_CHEM_HO_MASS;

mass[GAMGI_CHEM_ER] = GAMGI_CHEM_ER_MASS;

mass[GAMGI_CHEM_TM] = GAMGI_CHEM_TM_MASS;

mass[GAMGI_CHEM_YB] = GAMGI_CHEM_YB_MASS;

mass[GAMGI_CHEM_LU] = GAMGI_CHEM_LU_MASS;

mass[GAMGI_CHEM_HF] = GAMGI_CHEM_HF_MASS;

mass[GAMGI_CHEM_TA] = GAMGI_CHEM_TA_MASS;

mass[GAMGI_CHEM_W] = GAMGI_CHEM_W_MASS;

mass[GAMGI_CHEM_RE] = GAMGI_CHEM_RE_MASS;

mass[GAMGI_CHEM_OS] = GAMGI_CHEM_OS_MASS;

mass[GAMGI_CHEM_IR] = GAMGI_CHEM_IR_MASS;

mass[GAMGI_CHEM_PT] = GAMGI_CHEM_PT_MASS;

mass[GAMGI_CHEM_AU] = GAMGI_CHEM_AU_MASS;

mass[GAMGI_CHEM_HG] = GAMGI_CHEM_HG_MASS;

mass[GAMGI_CHEM_TL] = GAMGI_CHEM_TL_MASS;

mass[GAMGI_CHEM_PB] = GAMGI_CHEM_PB_MASS;

mass[GAMGI_CHEM_BI] = GAMGI_CHEM_BI_MASS;

mass[GAMGI_CHEM_PO] = GAMGI_CHEM_PO_MASS;

mass[GAMGI_CHEM_AT] = GAMGI_CHEM_AT_MASS;

mass[GAMGI_CHEM_RN] = GAMGI_CHEM_RN_MASS;

mass[GAMGI_CHEM_FR] = GAMGI_CHEM_FR_MASS;

mass[GAMGI_CHEM_RA] = GAMGI_CHEM_RA_MASS;

mass[GAMGI_CHEM_AC] = GAMGI_CHEM_AC_MASS;

mass[GAMGI_CHEM_TH] = GAMGI_CHEM_TH_MASS;

mass[GAMGI_CHEM_PA] = GAMGI_CHEM_PA_MASS;

mass[GAMGI_CHEM_U] = GAMGI_CHEM_U_MASS;

mass[GAMGI_CHEM_NP] = GAMGI_CHEM_NP_MASS;

mass[GAMGI_CHEM_PU] = GAMGI_CHEM_PU_MASS;

mass[GAMGI_CHEM_AM] = GAMGI_CHEM_AM_MASS;

mass[GAMGI_CHEM_CM] = GAMGI_CHEM_CM_MASS;

mass[GAMGI_CHEM_BK] = GAMGI_CHEM_BK_MASS;

mass[GAMGI_CHEM_CF] = GAMGI_CHEM_CF_MASS;

mass[GAMGI_CHEM_ES] = GAMGI_CHEM_ES_MASS;

mass[GAMGI_CHEM_FM] = GAMGI_CHEM_FM_MASS;

mass[GAMGI_CHEM_MD] = GAMGI_CHEM_MD_MASS;

mass[GAMGI_CHEM_NO] = GAMGI_CHEM_NO_MASS;

mass[GAMGI_CHEM_LR] = GAMGI_CHEM_LR_MASS;

mass[GAMGI_CHEM_RF] = GAMGI_CHEM_RF_MASS;

mass[GAMGI_CHEM_DB] = GAMGI_CHEM_DB_MASS;

mass[GAMGI_CHEM_SG] = GAMGI_CHEM_SG_MASS;

mass[GAMGI_CHEM_BH] = GAMGI_CHEM_BH_MASS;

mass[GAMGI_CHEM_HS] = GAMGI_CHEM_HS_MASS;

mass[GAMGI_CHEM_MT] = GAMGI_CHEM_MT_MASS;

mass[GAMGI_CHEM_DS] = GAMGI_CHEM_DS_MASS;

mass[GAMGI_CHEM_RG] = GAMGI_CHEM_RG_MASS;
}

/********** external function **********
 *                                     *
 *       GAMGI_CHEM_ATOM_RADIUS        *
 *                                     *
 * Set default radius for each element *
 ***************************************/

void gamgi_chem_atom_radius (gamgi_atom_class *atom_class)
{
double *radius = atom_class->radius;

radius[GAMGI_CHEM_DU] = GAMGI_CHEM_DU_RADIUS;

radius[GAMGI_CHEM_H] = GAMGI_CHEM_H_RADIUS;

radius[GAMGI_CHEM_HE] = GAMGI_CHEM_HE_RADIUS;

radius[GAMGI_CHEM_LI] = GAMGI_CHEM_LI_RADIUS;

radius[GAMGI_CHEM_BE] = GAMGI_CHEM_BE_RADIUS;

radius[GAMGI_CHEM_B] = GAMGI_CHEM_B_RADIUS;

radius[GAMGI_CHEM_C] = GAMGI_CHEM_C_RADIUS;

radius[GAMGI_CHEM_N] = GAMGI_CHEM_N_RADIUS;

radius[GAMGI_CHEM_O] = GAMGI_CHEM_O_RADIUS;

radius[GAMGI_CHEM_F] = GAMGI_CHEM_F_RADIUS;

radius[GAMGI_CHEM_NE] = GAMGI_CHEM_NE_RADIUS;

radius[GAMGI_CHEM_NA] = GAMGI_CHEM_NA_RADIUS;

radius[GAMGI_CHEM_MG] = GAMGI_CHEM_MG_RADIUS;

radius[GAMGI_CHEM_AL] = GAMGI_CHEM_AL_RADIUS;

radius[GAMGI_CHEM_SI] = GAMGI_CHEM_SI_RADIUS;
  
radius[GAMGI_CHEM_P] = GAMGI_CHEM_P_RADIUS;

radius[GAMGI_CHEM_S] = GAMGI_CHEM_S_RADIUS;

radius[GAMGI_CHEM_CL] = GAMGI_CHEM_CL_RADIUS;

radius[GAMGI_CHEM_AR] = GAMGI_CHEM_AR_RADIUS;

radius[GAMGI_CHEM_K] = GAMGI_CHEM_K_RADIUS;

radius[GAMGI_CHEM_CA] = GAMGI_CHEM_CA_RADIUS;
  
radius[GAMGI_CHEM_SC] = GAMGI_CHEM_SC_RADIUS;

radius[GAMGI_CHEM_TI] = GAMGI_CHEM_TI_RADIUS;

radius[GAMGI_CHEM_V] = GAMGI_CHEM_V_RADIUS;

radius[GAMGI_CHEM_CR] = GAMGI_CHEM_CR_RADIUS;

radius[GAMGI_CHEM_MN] = GAMGI_CHEM_MN_RADIUS;

radius[GAMGI_CHEM_FE] = GAMGI_CHEM_FE_RADIUS;
  
radius[GAMGI_CHEM_CO] = GAMGI_CHEM_CO_RADIUS;

radius[GAMGI_CHEM_NI] = GAMGI_CHEM_NI_RADIUS;

radius[GAMGI_CHEM_CU] = GAMGI_CHEM_CU_RADIUS;

radius[GAMGI_CHEM_ZN] = GAMGI_CHEM_ZN_RADIUS;

radius[GAMGI_CHEM_GA] = GAMGI_CHEM_GA_RADIUS;

radius[GAMGI_CHEM_GE] = GAMGI_CHEM_GE_RADIUS;
  
radius[GAMGI_CHEM_AS] = GAMGI_CHEM_AS_RADIUS;

radius[GAMGI_CHEM_SE] = GAMGI_CHEM_SE_RADIUS;

radius[GAMGI_CHEM_BR] = GAMGI_CHEM_BR_RADIUS;

radius[GAMGI_CHEM_KR] = GAMGI_CHEM_KR_RADIUS;

radius[GAMGI_CHEM_RB] = GAMGI_CHEM_RB_RADIUS;

radius[GAMGI_CHEM_SR] = GAMGI_CHEM_SR_RADIUS;
  
radius[GAMGI_CHEM_Y] = GAMGI_CHEM_Y_RADIUS;

radius[GAMGI_CHEM_ZR] = GAMGI_CHEM_ZR_RADIUS;

radius[GAMGI_CHEM_NB] = GAMGI_CHEM_NB_RADIUS;

radius[GAMGI_CHEM_MO] = GAMGI_CHEM_MO_RADIUS;

radius[GAMGI_CHEM_TC] = GAMGI_CHEM_TC_RADIUS;

radius[GAMGI_CHEM_RU] = GAMGI_CHEM_RU_RADIUS;
  
radius[GAMGI_CHEM_RH] = GAMGI_CHEM_RH_RADIUS;

radius[GAMGI_CHEM_PD] = GAMGI_CHEM_PD_RADIUS;

radius[GAMGI_CHEM_AG] = GAMGI_CHEM_AG_RADIUS;

radius[GAMGI_CHEM_CD] = GAMGI_CHEM_CD_RADIUS;

radius[GAMGI_CHEM_IN] = GAMGI_CHEM_IN_RADIUS;

radius[GAMGI_CHEM_SN] = GAMGI_CHEM_SN_RADIUS;
  
radius[GAMGI_CHEM_SB] = GAMGI_CHEM_SB_RADIUS;

radius[GAMGI_CHEM_TE] = GAMGI_CHEM_TE_RADIUS;

radius[GAMGI_CHEM_I] = GAMGI_CHEM_I_RADIUS;

radius[GAMGI_CHEM_XE] = GAMGI_CHEM_XE_RADIUS;

radius[GAMGI_CHEM_CS] = GAMGI_CHEM_CS_RADIUS;

radius[GAMGI_CHEM_BA] = GAMGI_CHEM_BA_RADIUS;

radius[GAMGI_CHEM_LA] = GAMGI_CHEM_LA_RADIUS;

radius[GAMGI_CHEM_CE] = GAMGI_CHEM_CE_RADIUS;

radius[GAMGI_CHEM_PR] = GAMGI_CHEM_PR_RADIUS;

radius[GAMGI_CHEM_ND] = GAMGI_CHEM_ND_RADIUS;

radius[GAMGI_CHEM_PM] = GAMGI_CHEM_PM_RADIUS;

radius[GAMGI_CHEM_SM] = GAMGI_CHEM_SM_RADIUS;

radius[GAMGI_CHEM_EU] = GAMGI_CHEM_EU_RADIUS;

radius[GAMGI_CHEM_GD] = GAMGI_CHEM_GD_RADIUS;

radius[GAMGI_CHEM_TB] = GAMGI_CHEM_TB_RADIUS;

radius[GAMGI_CHEM_DY] = GAMGI_CHEM_DY_RADIUS;

radius[GAMGI_CHEM_HO] = GAMGI_CHEM_HO_RADIUS;

radius[GAMGI_CHEM_ER] = GAMGI_CHEM_ER_RADIUS;

radius[GAMGI_CHEM_TM] = GAMGI_CHEM_TM_RADIUS;

radius[GAMGI_CHEM_YB] = GAMGI_CHEM_YB_RADIUS;

radius[GAMGI_CHEM_LU] = GAMGI_CHEM_LU_RADIUS;

radius[GAMGI_CHEM_HF] = GAMGI_CHEM_HF_RADIUS;

radius[GAMGI_CHEM_TA] = GAMGI_CHEM_TA_RADIUS;

radius[GAMGI_CHEM_W] = GAMGI_CHEM_W_RADIUS;

radius[GAMGI_CHEM_RE] = GAMGI_CHEM_RE_RADIUS;

radius[GAMGI_CHEM_OS] = GAMGI_CHEM_OS_RADIUS;

radius[GAMGI_CHEM_IR] = GAMGI_CHEM_IR_RADIUS;

radius[GAMGI_CHEM_PT] = GAMGI_CHEM_PT_RADIUS;

radius[GAMGI_CHEM_AU] = GAMGI_CHEM_AU_RADIUS;

radius[GAMGI_CHEM_HG] = GAMGI_CHEM_HG_RADIUS;

radius[GAMGI_CHEM_TL] = GAMGI_CHEM_TL_RADIUS;

radius[GAMGI_CHEM_PB] = GAMGI_CHEM_PB_RADIUS;

radius[GAMGI_CHEM_BI] = GAMGI_CHEM_BI_RADIUS;

radius[GAMGI_CHEM_PO] = GAMGI_CHEM_PO_RADIUS;

radius[GAMGI_CHEM_AT] = GAMGI_CHEM_AT_RADIUS;

radius[GAMGI_CHEM_RN] = GAMGI_CHEM_RN_RADIUS;

radius[GAMGI_CHEM_FR] = GAMGI_CHEM_FR_RADIUS;

radius[GAMGI_CHEM_RA] = GAMGI_CHEM_RA_RADIUS;

radius[GAMGI_CHEM_AC] = GAMGI_CHEM_AC_RADIUS;

radius[GAMGI_CHEM_TH] = GAMGI_CHEM_TH_RADIUS;

radius[GAMGI_CHEM_PA] = GAMGI_CHEM_PA_RADIUS;

radius[GAMGI_CHEM_U] = GAMGI_CHEM_U_RADIUS;

radius[GAMGI_CHEM_NP] = GAMGI_CHEM_NP_RADIUS;

radius[GAMGI_CHEM_PU] = GAMGI_CHEM_PU_RADIUS;

radius[GAMGI_CHEM_AM] = GAMGI_CHEM_AM_RADIUS;

radius[GAMGI_CHEM_CM] = GAMGI_CHEM_CM_RADIUS;

radius[GAMGI_CHEM_BK] = GAMGI_CHEM_BK_RADIUS;

radius[GAMGI_CHEM_CF] = GAMGI_CHEM_CF_RADIUS;

radius[GAMGI_CHEM_ES] = GAMGI_CHEM_ES_RADIUS;

radius[GAMGI_CHEM_FM] = GAMGI_CHEM_FM_RADIUS;

radius[GAMGI_CHEM_MD] = GAMGI_CHEM_MD_RADIUS;

radius[GAMGI_CHEM_NO] = GAMGI_CHEM_NO_RADIUS;

radius[GAMGI_CHEM_LR] = GAMGI_CHEM_LR_RADIUS;

radius[GAMGI_CHEM_RF] = GAMGI_CHEM_RF_RADIUS;

radius[GAMGI_CHEM_DB] = GAMGI_CHEM_DB_RADIUS;

radius[GAMGI_CHEM_SG] = GAMGI_CHEM_SG_RADIUS;

radius[GAMGI_CHEM_BH] = GAMGI_CHEM_BH_RADIUS;

radius[GAMGI_CHEM_HS] = GAMGI_CHEM_HS_RADIUS;

radius[GAMGI_CHEM_MT] = GAMGI_CHEM_MT_RADIUS;

radius[GAMGI_CHEM_DS] = GAMGI_CHEM_DS_RADIUS;

radius[GAMGI_CHEM_RG] = GAMGI_CHEM_RG_RADIUS;
}

void gamgi_chem_atom_color (gamgi_atom_class *atom_class)
{
float *red = atom_class->red;
float *green = atom_class->green;
float *blue = atom_class->blue;

red[GAMGI_CHEM_DU] = GAMGI_CHEM_DU_R;
green[GAMGI_CHEM_DU] = GAMGI_CHEM_DU_G;
blue[GAMGI_CHEM_DU] = GAMGI_CHEM_DU_B;

red[GAMGI_CHEM_H] = GAMGI_CHEM_H_R;
green[GAMGI_CHEM_H] = GAMGI_CHEM_H_G;
blue[GAMGI_CHEM_H] = GAMGI_CHEM_H_B;

red[GAMGI_CHEM_HE] = GAMGI_CHEM_HE_R;
green[GAMGI_CHEM_HE] = GAMGI_CHEM_HE_G;
blue[GAMGI_CHEM_HE] = GAMGI_CHEM_HE_B;

red[GAMGI_CHEM_LI] = GAMGI_CHEM_LI_R;
green[GAMGI_CHEM_LI] = GAMGI_CHEM_LI_G;
blue[GAMGI_CHEM_LI] = GAMGI_CHEM_LI_B;

red[GAMGI_CHEM_BE] = GAMGI_CHEM_BE_R;
green[GAMGI_CHEM_BE] = GAMGI_CHEM_BE_G;
blue[GAMGI_CHEM_BE] = GAMGI_CHEM_BE_B;

red[GAMGI_CHEM_B] = GAMGI_CHEM_B_R;
green[GAMGI_CHEM_B] = GAMGI_CHEM_B_G;
blue[GAMGI_CHEM_B] = GAMGI_CHEM_B_B;

red[GAMGI_CHEM_C] = GAMGI_CHEM_C_R;
green[GAMGI_CHEM_C] = GAMGI_CHEM_C_G;
blue[GAMGI_CHEM_C] = GAMGI_CHEM_C_B;

red[GAMGI_CHEM_N] = GAMGI_CHEM_N_R;
green[GAMGI_CHEM_N] = GAMGI_CHEM_N_G;
blue[GAMGI_CHEM_N] = GAMGI_CHEM_N_B;

red[GAMGI_CHEM_O] = GAMGI_CHEM_O_R;
green[GAMGI_CHEM_O] = GAMGI_CHEM_O_G;
blue[GAMGI_CHEM_O] = GAMGI_CHEM_O_B;

red[GAMGI_CHEM_F] = GAMGI_CHEM_F_R;
green[GAMGI_CHEM_F] = GAMGI_CHEM_F_G;
blue[GAMGI_CHEM_F] = GAMGI_CHEM_F_B;

red[GAMGI_CHEM_NE] = GAMGI_CHEM_NE_R;
green[GAMGI_CHEM_NE] = GAMGI_CHEM_NE_G;
blue[GAMGI_CHEM_NE] = GAMGI_CHEM_NE_B;

red[GAMGI_CHEM_NA] = GAMGI_CHEM_NA_R;
green[GAMGI_CHEM_NA] = GAMGI_CHEM_NA_G;
blue[GAMGI_CHEM_NA] = GAMGI_CHEM_NA_B;

red[GAMGI_CHEM_MG] = GAMGI_CHEM_MG_R;
green[GAMGI_CHEM_MG] = GAMGI_CHEM_MG_G;
blue[GAMGI_CHEM_MG] = GAMGI_CHEM_MG_B;

red[GAMGI_CHEM_AL] = GAMGI_CHEM_AL_R;
green[GAMGI_CHEM_AL] = GAMGI_CHEM_AL_G;
blue[GAMGI_CHEM_AL] = GAMGI_CHEM_AL_B;

red[GAMGI_CHEM_SI] = GAMGI_CHEM_SI_R;
green[GAMGI_CHEM_SI] = GAMGI_CHEM_SI_G;
blue[GAMGI_CHEM_SI] = GAMGI_CHEM_SI_B;
  
red[GAMGI_CHEM_P] = GAMGI_CHEM_P_R;
green[GAMGI_CHEM_P] = GAMGI_CHEM_P_G;
blue[GAMGI_CHEM_P] = GAMGI_CHEM_P_B;

red[GAMGI_CHEM_S] = GAMGI_CHEM_S_R;
green[GAMGI_CHEM_S] = GAMGI_CHEM_S_G;
blue[GAMGI_CHEM_S] = GAMGI_CHEM_S_B;

red[GAMGI_CHEM_CL] = GAMGI_CHEM_CL_R;
green[GAMGI_CHEM_CL] = GAMGI_CHEM_CL_G;
blue[GAMGI_CHEM_CL] = GAMGI_CHEM_CL_B;

red[GAMGI_CHEM_AR] = GAMGI_CHEM_AR_R;
green[GAMGI_CHEM_AR] = GAMGI_CHEM_AR_G;
blue[GAMGI_CHEM_AR] = GAMGI_CHEM_AR_B;

red[GAMGI_CHEM_K] = GAMGI_CHEM_K_R;
green[GAMGI_CHEM_K] = GAMGI_CHEM_K_G;
blue[GAMGI_CHEM_K] = GAMGI_CHEM_K_B;

red[GAMGI_CHEM_CA] = GAMGI_CHEM_CA_R;
green[GAMGI_CHEM_CA] = GAMGI_CHEM_CA_G;
blue[GAMGI_CHEM_CA] = GAMGI_CHEM_CA_B;
  
red[GAMGI_CHEM_SC] = GAMGI_CHEM_SC_R;
green[GAMGI_CHEM_SC] = GAMGI_CHEM_SC_G;
blue[GAMGI_CHEM_SC] = GAMGI_CHEM_SC_B;

red[GAMGI_CHEM_TI] = GAMGI_CHEM_TI_R;
green[GAMGI_CHEM_TI] = GAMGI_CHEM_TI_G;
blue[GAMGI_CHEM_TI] = GAMGI_CHEM_TI_B;

red[GAMGI_CHEM_V] = GAMGI_CHEM_V_R;
green[GAMGI_CHEM_V] = GAMGI_CHEM_V_G;
blue[GAMGI_CHEM_V] = GAMGI_CHEM_V_B;

red[GAMGI_CHEM_CR] = GAMGI_CHEM_CR_R;
green[GAMGI_CHEM_CR] = GAMGI_CHEM_CR_G;
blue[GAMGI_CHEM_CR] = GAMGI_CHEM_CR_B;

red[GAMGI_CHEM_MN] = GAMGI_CHEM_MN_R;
green[GAMGI_CHEM_MN] = GAMGI_CHEM_MN_G;
blue[GAMGI_CHEM_MN] = GAMGI_CHEM_MN_B;

red[GAMGI_CHEM_FE] = GAMGI_CHEM_FE_R;
green[GAMGI_CHEM_FE] = GAMGI_CHEM_FE_G;
blue[GAMGI_CHEM_FE] = GAMGI_CHEM_FE_B;
  
red[GAMGI_CHEM_CO] = GAMGI_CHEM_CO_R;
green[GAMGI_CHEM_CO] = GAMGI_CHEM_CO_G;
blue[GAMGI_CHEM_CO] = GAMGI_CHEM_CO_B;

red[GAMGI_CHEM_NI] = GAMGI_CHEM_NI_R;
green[GAMGI_CHEM_NI] = GAMGI_CHEM_NI_G;
blue[GAMGI_CHEM_NI] = GAMGI_CHEM_NI_B;

red[GAMGI_CHEM_CU] = GAMGI_CHEM_CU_R;
green[GAMGI_CHEM_CU] = GAMGI_CHEM_CU_G;
blue[GAMGI_CHEM_CU] = GAMGI_CHEM_CU_B;

red[GAMGI_CHEM_ZN] = GAMGI_CHEM_ZN_R;
green[GAMGI_CHEM_ZN] = GAMGI_CHEM_ZN_G;
blue[GAMGI_CHEM_ZN] = GAMGI_CHEM_ZN_B;

red[GAMGI_CHEM_GA] = GAMGI_CHEM_GA_R;
green[GAMGI_CHEM_GA] = GAMGI_CHEM_GA_G;
blue[GAMGI_CHEM_GA] = GAMGI_CHEM_GA_B;

red[GAMGI_CHEM_GE] = GAMGI_CHEM_GE_R;
green[GAMGI_CHEM_GE] = GAMGI_CHEM_GE_G;
blue[GAMGI_CHEM_GE] = GAMGI_CHEM_GE_B;
  
red[GAMGI_CHEM_AS] = GAMGI_CHEM_AS_R;
green[GAMGI_CHEM_AS] = GAMGI_CHEM_AS_G;
blue[GAMGI_CHEM_AS] = GAMGI_CHEM_AS_B;

red[GAMGI_CHEM_SE] = GAMGI_CHEM_SE_R;
green[GAMGI_CHEM_SE] = GAMGI_CHEM_SE_G;
blue[GAMGI_CHEM_SE] = GAMGI_CHEM_SE_B;

red[GAMGI_CHEM_BR] = GAMGI_CHEM_BR_R;
green[GAMGI_CHEM_BR] = GAMGI_CHEM_BR_G;
blue[GAMGI_CHEM_BR] = GAMGI_CHEM_BR_B;

red[GAMGI_CHEM_KR] = GAMGI_CHEM_KR_R;
green[GAMGI_CHEM_KR] = GAMGI_CHEM_KR_G;
blue[GAMGI_CHEM_KR] = GAMGI_CHEM_KR_B;

red[GAMGI_CHEM_RB] = GAMGI_CHEM_RB_R;
green[GAMGI_CHEM_RB] = GAMGI_CHEM_RB_G;
blue[GAMGI_CHEM_RB] = GAMGI_CHEM_RB_B;

red[GAMGI_CHEM_SR] = GAMGI_CHEM_SR_R;
green[GAMGI_CHEM_SR] = GAMGI_CHEM_SR_G;
blue[GAMGI_CHEM_SR] = GAMGI_CHEM_SR_B;
  
red[GAMGI_CHEM_Y] = GAMGI_CHEM_Y_R;
green[GAMGI_CHEM_Y] = GAMGI_CHEM_Y_G;
blue[GAMGI_CHEM_Y] = GAMGI_CHEM_Y_B;

red[GAMGI_CHEM_ZR] = GAMGI_CHEM_ZR_R;
green[GAMGI_CHEM_ZR] = GAMGI_CHEM_ZR_G;
blue[GAMGI_CHEM_ZR] = GAMGI_CHEM_ZR_B;

red[GAMGI_CHEM_NB] = GAMGI_CHEM_NB_R;
green[GAMGI_CHEM_NB] = GAMGI_CHEM_NB_G;
blue[GAMGI_CHEM_NB] = GAMGI_CHEM_NB_B;

red[GAMGI_CHEM_MO] = GAMGI_CHEM_MO_R;
green[GAMGI_CHEM_MO] = GAMGI_CHEM_MO_G;
blue[GAMGI_CHEM_MO] = GAMGI_CHEM_MO_B;

red[GAMGI_CHEM_TC] = GAMGI_CHEM_TC_R;
green[GAMGI_CHEM_TC] = GAMGI_CHEM_TC_G;
blue[GAMGI_CHEM_TC] = GAMGI_CHEM_TC_B;

red[GAMGI_CHEM_RU] = GAMGI_CHEM_RU_R;
green[GAMGI_CHEM_RU] = GAMGI_CHEM_RU_G;
blue[GAMGI_CHEM_RU] = GAMGI_CHEM_RU_B;
  
red[GAMGI_CHEM_RH] = GAMGI_CHEM_RH_R;
green[GAMGI_CHEM_RH] = GAMGI_CHEM_RH_G;
blue[GAMGI_CHEM_RH] = GAMGI_CHEM_RH_B;

red[GAMGI_CHEM_PD] = GAMGI_CHEM_PD_R;
green[GAMGI_CHEM_PD] = GAMGI_CHEM_PD_G;
blue[GAMGI_CHEM_PD] = GAMGI_CHEM_PD_B;

red[GAMGI_CHEM_AG] = GAMGI_CHEM_AG_R;
green[GAMGI_CHEM_AG] = GAMGI_CHEM_AG_G;
blue[GAMGI_CHEM_AG] = GAMGI_CHEM_AG_B;

red[GAMGI_CHEM_CD] = GAMGI_CHEM_CD_R;
green[GAMGI_CHEM_CD] = GAMGI_CHEM_CD_G;
blue[GAMGI_CHEM_CD] = GAMGI_CHEM_CD_B;

red[GAMGI_CHEM_IN] = GAMGI_CHEM_IN_R;
green[GAMGI_CHEM_IN] = GAMGI_CHEM_IN_G;
blue[GAMGI_CHEM_IN] = GAMGI_CHEM_IN_B;

red[GAMGI_CHEM_SN] = GAMGI_CHEM_SN_R;
green[GAMGI_CHEM_SN] = GAMGI_CHEM_SN_G;
blue[GAMGI_CHEM_SN] = GAMGI_CHEM_SN_B;
  
red[GAMGI_CHEM_SB] = GAMGI_CHEM_SB_R;
green[GAMGI_CHEM_SB] = GAMGI_CHEM_SB_G;
blue[GAMGI_CHEM_SB] = GAMGI_CHEM_SB_B;

red[GAMGI_CHEM_TE] = GAMGI_CHEM_TE_R;
green[GAMGI_CHEM_TE] = GAMGI_CHEM_TE_G;
blue[GAMGI_CHEM_TE] = GAMGI_CHEM_TE_B;

red[GAMGI_CHEM_I] = GAMGI_CHEM_I_R;
green[GAMGI_CHEM_I] = GAMGI_CHEM_I_G;
blue[GAMGI_CHEM_I] = GAMGI_CHEM_I_B;

red[GAMGI_CHEM_XE] = GAMGI_CHEM_XE_R;
green[GAMGI_CHEM_XE] = GAMGI_CHEM_XE_G;
blue[GAMGI_CHEM_XE] = GAMGI_CHEM_XE_B;

red[GAMGI_CHEM_CS] = GAMGI_CHEM_CS_R;
green[GAMGI_CHEM_CS] = GAMGI_CHEM_CS_G;
blue[GAMGI_CHEM_CS] = GAMGI_CHEM_CS_B;

red[GAMGI_CHEM_BA] = GAMGI_CHEM_BA_R;
green[GAMGI_CHEM_BA] = GAMGI_CHEM_BA_G;
blue[GAMGI_CHEM_BA] = GAMGI_CHEM_BA_B;

red[GAMGI_CHEM_LA] = GAMGI_CHEM_LA_R;
green[GAMGI_CHEM_LA] = GAMGI_CHEM_LA_G;
blue[GAMGI_CHEM_LA] = GAMGI_CHEM_LA_B;

red[GAMGI_CHEM_CE] = GAMGI_CHEM_CE_R;
green[GAMGI_CHEM_CE] = GAMGI_CHEM_CE_G;
blue[GAMGI_CHEM_CE] = GAMGI_CHEM_CE_B;

red[GAMGI_CHEM_PR] = GAMGI_CHEM_PR_R;
green[GAMGI_CHEM_PR] = GAMGI_CHEM_PR_G;
blue[GAMGI_CHEM_PR] = GAMGI_CHEM_PR_B;

red[GAMGI_CHEM_ND] = GAMGI_CHEM_ND_R;
green[GAMGI_CHEM_ND] = GAMGI_CHEM_ND_G;
blue[GAMGI_CHEM_ND] = GAMGI_CHEM_ND_B;

red[GAMGI_CHEM_PM] = GAMGI_CHEM_PM_R;
green[GAMGI_CHEM_PM] = GAMGI_CHEM_PM_G;
blue[GAMGI_CHEM_PM] = GAMGI_CHEM_PM_B;

red[GAMGI_CHEM_SM] = GAMGI_CHEM_SM_R;
green[GAMGI_CHEM_SM] = GAMGI_CHEM_SM_G;
blue[GAMGI_CHEM_SM] = GAMGI_CHEM_SM_B;

red[GAMGI_CHEM_EU] = GAMGI_CHEM_EU_R;
green[GAMGI_CHEM_EU] = GAMGI_CHEM_EU_G;
blue[GAMGI_CHEM_EU] = GAMGI_CHEM_EU_B;

red[GAMGI_CHEM_GD] = GAMGI_CHEM_GD_R;
green[GAMGI_CHEM_GD] = GAMGI_CHEM_GD_G;
blue[GAMGI_CHEM_GD] = GAMGI_CHEM_GD_B;

red[GAMGI_CHEM_TB] = GAMGI_CHEM_TB_R;
green[GAMGI_CHEM_TB] = GAMGI_CHEM_TB_G;
blue[GAMGI_CHEM_TB] = GAMGI_CHEM_TB_B;

red[GAMGI_CHEM_DY] = GAMGI_CHEM_DY_R;
green[GAMGI_CHEM_DY] = GAMGI_CHEM_DY_G;
blue[GAMGI_CHEM_DY] = GAMGI_CHEM_DY_B;

red[GAMGI_CHEM_HO] = GAMGI_CHEM_HO_R;
green[GAMGI_CHEM_HO] = GAMGI_CHEM_HO_G;
blue[GAMGI_CHEM_HO] = GAMGI_CHEM_HO_B;

red[GAMGI_CHEM_ER] = GAMGI_CHEM_ER_R;
green[GAMGI_CHEM_ER] = GAMGI_CHEM_ER_G;
blue[GAMGI_CHEM_ER] = GAMGI_CHEM_ER_B;

red[GAMGI_CHEM_TM] = GAMGI_CHEM_TM_R;
green[GAMGI_CHEM_TM] = GAMGI_CHEM_TM_G;
blue[GAMGI_CHEM_TM] = GAMGI_CHEM_TM_B;

red[GAMGI_CHEM_YB] = GAMGI_CHEM_YB_R;
green[GAMGI_CHEM_YB] = GAMGI_CHEM_YB_G;
blue[GAMGI_CHEM_YB] = GAMGI_CHEM_YB_B;

red[GAMGI_CHEM_LU] = GAMGI_CHEM_LU_R;
green[GAMGI_CHEM_LU] = GAMGI_CHEM_LU_G;
blue[GAMGI_CHEM_LU] = GAMGI_CHEM_LU_B;

red[GAMGI_CHEM_HF] = GAMGI_CHEM_HF_R;
green[GAMGI_CHEM_HF] = GAMGI_CHEM_HF_G;
blue[GAMGI_CHEM_HF] = GAMGI_CHEM_HF_B;

red[GAMGI_CHEM_TA] = GAMGI_CHEM_TA_R;
green[GAMGI_CHEM_TA] = GAMGI_CHEM_TA_G;
blue[GAMGI_CHEM_TA] = GAMGI_CHEM_TA_B;

red[GAMGI_CHEM_W] = GAMGI_CHEM_W_R;
green[GAMGI_CHEM_W] = GAMGI_CHEM_W_G;
blue[GAMGI_CHEM_W] = GAMGI_CHEM_W_B;

red[GAMGI_CHEM_RE] = GAMGI_CHEM_RE_R;
green[GAMGI_CHEM_RE] = GAMGI_CHEM_RE_G;
blue[GAMGI_CHEM_RE] = GAMGI_CHEM_RE_B;

red[GAMGI_CHEM_OS] = GAMGI_CHEM_OS_R;
green[GAMGI_CHEM_OS] = GAMGI_CHEM_OS_G;
blue[GAMGI_CHEM_OS] = GAMGI_CHEM_OS_B;

red[GAMGI_CHEM_IR] = GAMGI_CHEM_IR_R;
green[GAMGI_CHEM_IR] = GAMGI_CHEM_IR_G;
blue[GAMGI_CHEM_IR] = GAMGI_CHEM_IR_B;

red[GAMGI_CHEM_PT] = GAMGI_CHEM_PT_R;
green[GAMGI_CHEM_PT] = GAMGI_CHEM_PT_G;
blue[GAMGI_CHEM_PT] = GAMGI_CHEM_PT_B;

red[GAMGI_CHEM_AU] = GAMGI_CHEM_AU_R;
green[GAMGI_CHEM_AU] = GAMGI_CHEM_AU_G;
blue[GAMGI_CHEM_AU] = GAMGI_CHEM_AU_B;

red[GAMGI_CHEM_HG] = GAMGI_CHEM_HG_R;
green[GAMGI_CHEM_HG] = GAMGI_CHEM_HG_G;
blue[GAMGI_CHEM_HG] = GAMGI_CHEM_HG_B;

red[GAMGI_CHEM_TL] = GAMGI_CHEM_TL_R;
green[GAMGI_CHEM_TL] = GAMGI_CHEM_TL_G;
blue[GAMGI_CHEM_TL] = GAMGI_CHEM_TL_B;

red[GAMGI_CHEM_PB] = GAMGI_CHEM_PB_R;
green[GAMGI_CHEM_PB] = GAMGI_CHEM_PB_G;
blue[GAMGI_CHEM_PB] = GAMGI_CHEM_PB_B;

red[GAMGI_CHEM_BI] = GAMGI_CHEM_BI_R;
green[GAMGI_CHEM_BI] = GAMGI_CHEM_BI_G;
blue[GAMGI_CHEM_BI] = GAMGI_CHEM_BI_B;

red[GAMGI_CHEM_PO] = GAMGI_CHEM_PO_R;
green[GAMGI_CHEM_PO] = GAMGI_CHEM_PO_G;
blue[GAMGI_CHEM_PO] = GAMGI_CHEM_PO_B;

red[GAMGI_CHEM_AT] = GAMGI_CHEM_AT_R;
green[GAMGI_CHEM_AT] = GAMGI_CHEM_AT_G;
blue[GAMGI_CHEM_AT] = GAMGI_CHEM_AT_B;

red[GAMGI_CHEM_RN] = GAMGI_CHEM_RN_R;
green[GAMGI_CHEM_RN] = GAMGI_CHEM_RN_G;
blue[GAMGI_CHEM_RN] = GAMGI_CHEM_RN_B;

red[GAMGI_CHEM_FR] = GAMGI_CHEM_FR_R;
green[GAMGI_CHEM_FR] = GAMGI_CHEM_FR_G;
blue[GAMGI_CHEM_FR] = GAMGI_CHEM_FR_B;

red[GAMGI_CHEM_RA] = GAMGI_CHEM_RA_R;
green[GAMGI_CHEM_RA] = GAMGI_CHEM_RA_G;
blue[GAMGI_CHEM_RA] = GAMGI_CHEM_RA_B;

red[GAMGI_CHEM_AC] = GAMGI_CHEM_AC_R;
green[GAMGI_CHEM_AC] = GAMGI_CHEM_AC_G;
blue[GAMGI_CHEM_AC] = GAMGI_CHEM_AC_B;

red[GAMGI_CHEM_TH] = GAMGI_CHEM_TH_R;
green[GAMGI_CHEM_TH] = GAMGI_CHEM_TH_G;
blue[GAMGI_CHEM_TH] = GAMGI_CHEM_TH_B;

red[GAMGI_CHEM_PA] = GAMGI_CHEM_PA_R;
green[GAMGI_CHEM_PA] = GAMGI_CHEM_PA_G;
blue[GAMGI_CHEM_PA] = GAMGI_CHEM_PA_B;

red[GAMGI_CHEM_U] = GAMGI_CHEM_U_R;
green[GAMGI_CHEM_U] = GAMGI_CHEM_U_G;
blue[GAMGI_CHEM_U] = GAMGI_CHEM_U_B;

red[GAMGI_CHEM_NP] = GAMGI_CHEM_NP_R;
green[GAMGI_CHEM_NP] = GAMGI_CHEM_NP_G;
blue[GAMGI_CHEM_NP] = GAMGI_CHEM_NP_B;

red[GAMGI_CHEM_PU] = GAMGI_CHEM_PU_R;
green[GAMGI_CHEM_PU] = GAMGI_CHEM_PU_G;
blue[GAMGI_CHEM_PU] = GAMGI_CHEM_PU_B;

red[GAMGI_CHEM_AM] = GAMGI_CHEM_AM_R;
green[GAMGI_CHEM_AM] = GAMGI_CHEM_AM_G;
blue[GAMGI_CHEM_AM] = GAMGI_CHEM_AM_B;

red[GAMGI_CHEM_CM] = GAMGI_CHEM_CM_R;
green[GAMGI_CHEM_CM] = GAMGI_CHEM_CM_G;
blue[GAMGI_CHEM_CM] = GAMGI_CHEM_CM_B;

red[GAMGI_CHEM_BK] = GAMGI_CHEM_BK_R;
green[GAMGI_CHEM_BK] = GAMGI_CHEM_BK_G;
blue[GAMGI_CHEM_BK] = GAMGI_CHEM_BK_B;

red[GAMGI_CHEM_CF] = GAMGI_CHEM_CF_R;
green[GAMGI_CHEM_CF] = GAMGI_CHEM_CF_G;
blue[GAMGI_CHEM_CF] = GAMGI_CHEM_CF_B;

red[GAMGI_CHEM_ES] = GAMGI_CHEM_ES_R;
green[GAMGI_CHEM_ES] = GAMGI_CHEM_ES_G;
blue[GAMGI_CHEM_ES] = GAMGI_CHEM_ES_B;

red[GAMGI_CHEM_FM] = GAMGI_CHEM_FM_R;
green[GAMGI_CHEM_FM] = GAMGI_CHEM_FM_G;
blue[GAMGI_CHEM_FM] = GAMGI_CHEM_FM_B;

red[GAMGI_CHEM_MD] = GAMGI_CHEM_MD_R;
green[GAMGI_CHEM_MD] = GAMGI_CHEM_MD_G;
blue[GAMGI_CHEM_MD] = GAMGI_CHEM_MD_B;

red[GAMGI_CHEM_NO] = GAMGI_CHEM_NO_R;
green[GAMGI_CHEM_NO] = GAMGI_CHEM_NO_G;
blue[GAMGI_CHEM_NO] = GAMGI_CHEM_NO_B;

red[GAMGI_CHEM_LR] = GAMGI_CHEM_LR_R;
green[GAMGI_CHEM_LR] = GAMGI_CHEM_LR_G;
blue[GAMGI_CHEM_LR] = GAMGI_CHEM_LR_B;

red[GAMGI_CHEM_RF] = GAMGI_CHEM_RF_R;
green[GAMGI_CHEM_RF] = GAMGI_CHEM_RF_G;
blue[GAMGI_CHEM_RF] = GAMGI_CHEM_RF_B;

red[GAMGI_CHEM_DB] = GAMGI_CHEM_DB_R;
green[GAMGI_CHEM_DB] = GAMGI_CHEM_DB_G;
blue[GAMGI_CHEM_DB] = GAMGI_CHEM_DB_B;

red[GAMGI_CHEM_SG] = GAMGI_CHEM_SG_R;
green[GAMGI_CHEM_SG] = GAMGI_CHEM_SG_G;
blue[GAMGI_CHEM_SG] = GAMGI_CHEM_SG_B;

red[GAMGI_CHEM_BH] = GAMGI_CHEM_BH_R;
green[GAMGI_CHEM_BH] = GAMGI_CHEM_BH_G;
blue[GAMGI_CHEM_BH] = GAMGI_CHEM_BH_B;

red[GAMGI_CHEM_HS] = GAMGI_CHEM_HS_R;
green[GAMGI_CHEM_HS] = GAMGI_CHEM_HS_G;
blue[GAMGI_CHEM_HS] = GAMGI_CHEM_HS_B;

red[GAMGI_CHEM_MT] = GAMGI_CHEM_MT_R;
green[GAMGI_CHEM_MT] = GAMGI_CHEM_MT_G;
blue[GAMGI_CHEM_MT] = GAMGI_CHEM_MT_B;

red[GAMGI_CHEM_DS] = GAMGI_CHEM_DS_R;
green[GAMGI_CHEM_DS] = GAMGI_CHEM_DS_G;
blue[GAMGI_CHEM_DS] = GAMGI_CHEM_DS_B;

red[GAMGI_CHEM_RG] = GAMGI_CHEM_RG_R;
green[GAMGI_CHEM_RG] = GAMGI_CHEM_RG_G;
blue[GAMGI_CHEM_RG] = GAMGI_CHEM_RG_B;
}

void gamgi_chem_atom_export (gamgi_window *window, FILE *fp,
int depth_parent, int *depth_last, gamgi_bool *error)
{
gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error, 
GAMGI_CHEM_H, GAMGI_CHEM_H_MASS, GAMGI_CHEM_H_RADIUS, 
GAMGI_CHEM_H_R, GAMGI_CHEM_H_G, GAMGI_CHEM_H_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_HE, GAMGI_CHEM_HE_MASS, GAMGI_CHEM_HE_RADIUS,
GAMGI_CHEM_HE_R, GAMGI_CHEM_HE_G, GAMGI_CHEM_HE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_LI, GAMGI_CHEM_LI_MASS, GAMGI_CHEM_LI_RADIUS,
GAMGI_CHEM_LI_R, GAMGI_CHEM_LI_G, GAMGI_CHEM_LI_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_BE, GAMGI_CHEM_BE_MASS, GAMGI_CHEM_BE_RADIUS,
GAMGI_CHEM_BE_R, GAMGI_CHEM_BE_G, GAMGI_CHEM_BE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_B, GAMGI_CHEM_B_MASS, GAMGI_CHEM_B_RADIUS,
GAMGI_CHEM_B_R, GAMGI_CHEM_B_G, GAMGI_CHEM_B_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_C, GAMGI_CHEM_C_MASS, GAMGI_CHEM_C_RADIUS,
GAMGI_CHEM_C_R, GAMGI_CHEM_C_G, GAMGI_CHEM_C_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_N, GAMGI_CHEM_N_MASS, GAMGI_CHEM_N_RADIUS,
GAMGI_CHEM_N_R, GAMGI_CHEM_N_G, GAMGI_CHEM_N_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_O, GAMGI_CHEM_O_MASS, GAMGI_CHEM_O_RADIUS,
GAMGI_CHEM_O_R, GAMGI_CHEM_O_G, GAMGI_CHEM_O_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_F, GAMGI_CHEM_F_MASS, GAMGI_CHEM_F_RADIUS,
GAMGI_CHEM_F_R, GAMGI_CHEM_F_G, GAMGI_CHEM_F_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_NE, GAMGI_CHEM_NE_MASS, GAMGI_CHEM_NE_RADIUS,
GAMGI_CHEM_NE_R, GAMGI_CHEM_NE_G, GAMGI_CHEM_NE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_NA, GAMGI_CHEM_NA_MASS, GAMGI_CHEM_NA_RADIUS,
GAMGI_CHEM_NA_R, GAMGI_CHEM_NA_G, GAMGI_CHEM_NA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_MG, GAMGI_CHEM_MG_MASS, GAMGI_CHEM_MG_RADIUS,
GAMGI_CHEM_MG_R, GAMGI_CHEM_MG_G, GAMGI_CHEM_MG_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AL, GAMGI_CHEM_AL_MASS, GAMGI_CHEM_AL_RADIUS,
GAMGI_CHEM_AL_R, GAMGI_CHEM_AL_G, GAMGI_CHEM_AL_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SI, GAMGI_CHEM_SI_MASS, GAMGI_CHEM_SI_RADIUS,
GAMGI_CHEM_SI_R, GAMGI_CHEM_SI_G, GAMGI_CHEM_SI_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_P, GAMGI_CHEM_P_MASS, GAMGI_CHEM_P_RADIUS,
GAMGI_CHEM_P_R, GAMGI_CHEM_P_G, GAMGI_CHEM_P_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_S, GAMGI_CHEM_S_MASS, GAMGI_CHEM_S_RADIUS,
GAMGI_CHEM_S_R, GAMGI_CHEM_S_G, GAMGI_CHEM_S_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CL, GAMGI_CHEM_CL_MASS, GAMGI_CHEM_CL_RADIUS,
GAMGI_CHEM_CL_R, GAMGI_CHEM_CL_G, GAMGI_CHEM_CL_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AR, GAMGI_CHEM_AR_MASS, GAMGI_CHEM_AR_RADIUS,
GAMGI_CHEM_AR_R, GAMGI_CHEM_AR_G, GAMGI_CHEM_AR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_K, GAMGI_CHEM_K_MASS, GAMGI_CHEM_K_RADIUS,
GAMGI_CHEM_K_R, GAMGI_CHEM_K_G, GAMGI_CHEM_K_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CA, GAMGI_CHEM_CA_MASS, GAMGI_CHEM_CA_RADIUS,
GAMGI_CHEM_CA_R, GAMGI_CHEM_CA_G, GAMGI_CHEM_CA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SC, GAMGI_CHEM_SC_MASS, GAMGI_CHEM_SC_RADIUS,
GAMGI_CHEM_SC_R, GAMGI_CHEM_SC_G, GAMGI_CHEM_SC_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TI, GAMGI_CHEM_TI_MASS, GAMGI_CHEM_TI_RADIUS,
GAMGI_CHEM_TI_R, GAMGI_CHEM_TI_G, GAMGI_CHEM_TI_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_V, GAMGI_CHEM_V_MASS, GAMGI_CHEM_V_RADIUS,
GAMGI_CHEM_V_R, GAMGI_CHEM_V_G, GAMGI_CHEM_V_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CR, GAMGI_CHEM_CR_MASS, GAMGI_CHEM_CR_RADIUS,
GAMGI_CHEM_CR_R, GAMGI_CHEM_CR_G, GAMGI_CHEM_CR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_MN, GAMGI_CHEM_MN_MASS, GAMGI_CHEM_MN_RADIUS,
GAMGI_CHEM_MN_R, GAMGI_CHEM_MN_G, GAMGI_CHEM_MN_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_FE, GAMGI_CHEM_FE_MASS, GAMGI_CHEM_FE_RADIUS,
GAMGI_CHEM_FE_R, GAMGI_CHEM_FE_G, GAMGI_CHEM_FE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CO, GAMGI_CHEM_CO_MASS, GAMGI_CHEM_CO_RADIUS,
GAMGI_CHEM_CO_R, GAMGI_CHEM_CO_G, GAMGI_CHEM_CO_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_NI, GAMGI_CHEM_NI_MASS, GAMGI_CHEM_NI_RADIUS,
GAMGI_CHEM_NI_R, GAMGI_CHEM_NI_G, GAMGI_CHEM_NI_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CU, GAMGI_CHEM_CU_MASS, GAMGI_CHEM_CU_RADIUS,
GAMGI_CHEM_CU_R, GAMGI_CHEM_CU_G, GAMGI_CHEM_CU_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_ZN, GAMGI_CHEM_ZN_MASS, GAMGI_CHEM_ZN_RADIUS,
GAMGI_CHEM_ZN_R, GAMGI_CHEM_ZN_G, GAMGI_CHEM_ZN_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_GA, GAMGI_CHEM_GA_MASS, GAMGI_CHEM_GA_RADIUS,
GAMGI_CHEM_GA_R, GAMGI_CHEM_GA_G, GAMGI_CHEM_GA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_GE, GAMGI_CHEM_GE_MASS, GAMGI_CHEM_GE_RADIUS,
GAMGI_CHEM_GE_R, GAMGI_CHEM_GE_G, GAMGI_CHEM_GE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AS, GAMGI_CHEM_AS_MASS, GAMGI_CHEM_AS_RADIUS,
GAMGI_CHEM_AS_R, GAMGI_CHEM_AS_G, GAMGI_CHEM_AS_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SE, GAMGI_CHEM_SE_MASS, GAMGI_CHEM_SE_RADIUS,
GAMGI_CHEM_SE_R, GAMGI_CHEM_SE_G, GAMGI_CHEM_SE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_BR, GAMGI_CHEM_BR_MASS, GAMGI_CHEM_BR_RADIUS,
GAMGI_CHEM_BR_R, GAMGI_CHEM_BR_G, GAMGI_CHEM_BR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_KR, GAMGI_CHEM_KR_MASS, GAMGI_CHEM_KR_RADIUS,
GAMGI_CHEM_KR_R, GAMGI_CHEM_KR_G, GAMGI_CHEM_KR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RB, GAMGI_CHEM_RB_MASS, GAMGI_CHEM_RB_RADIUS,
GAMGI_CHEM_RB_R, GAMGI_CHEM_RB_G, GAMGI_CHEM_RB_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SR, GAMGI_CHEM_SR_MASS, GAMGI_CHEM_SR_RADIUS,
GAMGI_CHEM_SR_R, GAMGI_CHEM_SR_G, GAMGI_CHEM_SR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_Y, GAMGI_CHEM_Y_MASS, GAMGI_CHEM_Y_RADIUS,
GAMGI_CHEM_Y_R, GAMGI_CHEM_Y_G, GAMGI_CHEM_Y_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_ZR, GAMGI_CHEM_ZR_MASS, GAMGI_CHEM_ZR_RADIUS,
GAMGI_CHEM_ZR_R, GAMGI_CHEM_ZR_G, GAMGI_CHEM_ZR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_NB, GAMGI_CHEM_NB_MASS, GAMGI_CHEM_NB_RADIUS,
GAMGI_CHEM_NB_R, GAMGI_CHEM_NB_G, GAMGI_CHEM_NB_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_MO, GAMGI_CHEM_MO_MASS, GAMGI_CHEM_MO_RADIUS,
GAMGI_CHEM_MO_R, GAMGI_CHEM_MO_G, GAMGI_CHEM_MO_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TC, GAMGI_CHEM_TC_MASS, GAMGI_CHEM_TC_RADIUS,
GAMGI_CHEM_TC_R, GAMGI_CHEM_TC_G, GAMGI_CHEM_TC_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RU, GAMGI_CHEM_RU_MASS, GAMGI_CHEM_RU_RADIUS,
GAMGI_CHEM_RU_R, GAMGI_CHEM_RU_G, GAMGI_CHEM_RU_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RH, GAMGI_CHEM_RH_MASS, GAMGI_CHEM_RH_RADIUS,
GAMGI_CHEM_RH_R, GAMGI_CHEM_RH_G, GAMGI_CHEM_RH_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PD, GAMGI_CHEM_PD_MASS, GAMGI_CHEM_PD_RADIUS,
GAMGI_CHEM_PD_R, GAMGI_CHEM_PD_G, GAMGI_CHEM_PD_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AG, GAMGI_CHEM_AG_MASS, GAMGI_CHEM_AG_RADIUS,
GAMGI_CHEM_AG_R, GAMGI_CHEM_AG_G, GAMGI_CHEM_AG_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CD, GAMGI_CHEM_CD_MASS, GAMGI_CHEM_CD_RADIUS,
GAMGI_CHEM_CD_R, GAMGI_CHEM_CD_G, GAMGI_CHEM_CD_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_IN, GAMGI_CHEM_IN_MASS, GAMGI_CHEM_IN_RADIUS,
GAMGI_CHEM_IN_R, GAMGI_CHEM_IN_G, GAMGI_CHEM_IN_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SN, GAMGI_CHEM_SN_MASS, GAMGI_CHEM_SN_RADIUS,
GAMGI_CHEM_SN_R, GAMGI_CHEM_SN_G, GAMGI_CHEM_SN_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SB, GAMGI_CHEM_SB_MASS, GAMGI_CHEM_SB_RADIUS,
GAMGI_CHEM_SB_R, GAMGI_CHEM_SB_G, GAMGI_CHEM_SB_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TE, GAMGI_CHEM_TE_MASS, GAMGI_CHEM_TE_RADIUS,
GAMGI_CHEM_TE_R, GAMGI_CHEM_TE_G, GAMGI_CHEM_TE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_I, GAMGI_CHEM_I_MASS, GAMGI_CHEM_I_RADIUS,
GAMGI_CHEM_I_R, GAMGI_CHEM_I_G, GAMGI_CHEM_I_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_XE, GAMGI_CHEM_XE_MASS, GAMGI_CHEM_XE_RADIUS,
GAMGI_CHEM_XE_R, GAMGI_CHEM_XE_G, GAMGI_CHEM_XE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CS, GAMGI_CHEM_CS_MASS, GAMGI_CHEM_CS_RADIUS,
GAMGI_CHEM_CS_R, GAMGI_CHEM_CS_G, GAMGI_CHEM_CS_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_BA, GAMGI_CHEM_BA_MASS, GAMGI_CHEM_BA_RADIUS,
GAMGI_CHEM_BA_R, GAMGI_CHEM_BA_G, GAMGI_CHEM_BA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_LA, GAMGI_CHEM_LA_MASS, GAMGI_CHEM_LA_RADIUS,
GAMGI_CHEM_LA_R, GAMGI_CHEM_LA_G, GAMGI_CHEM_LA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CE, GAMGI_CHEM_CE_MASS, GAMGI_CHEM_CE_RADIUS,
GAMGI_CHEM_CE_R, GAMGI_CHEM_CE_G, GAMGI_CHEM_CE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PR, GAMGI_CHEM_PR_MASS, GAMGI_CHEM_PR_RADIUS,
GAMGI_CHEM_PR_R, GAMGI_CHEM_PR_G, GAMGI_CHEM_PR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_ND, GAMGI_CHEM_ND_MASS, GAMGI_CHEM_ND_RADIUS,
GAMGI_CHEM_ND_R, GAMGI_CHEM_ND_G, GAMGI_CHEM_ND_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PM, GAMGI_CHEM_PM_MASS, GAMGI_CHEM_PM_RADIUS,
GAMGI_CHEM_PM_R, GAMGI_CHEM_PM_G, GAMGI_CHEM_PM_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SM, GAMGI_CHEM_SM_MASS, GAMGI_CHEM_SM_RADIUS,
GAMGI_CHEM_SM_R, GAMGI_CHEM_SM_G, GAMGI_CHEM_SM_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_EU, GAMGI_CHEM_EU_MASS, GAMGI_CHEM_EU_RADIUS,
GAMGI_CHEM_EU_R, GAMGI_CHEM_EU_G, GAMGI_CHEM_EU_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_GD, GAMGI_CHEM_GD_MASS, GAMGI_CHEM_GD_RADIUS,
GAMGI_CHEM_GD_R, GAMGI_CHEM_GD_G, GAMGI_CHEM_GD_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TB, GAMGI_CHEM_TB_MASS, GAMGI_CHEM_TB_RADIUS,
GAMGI_CHEM_TB_R, GAMGI_CHEM_TB_G, GAMGI_CHEM_TB_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_DY, GAMGI_CHEM_DY_MASS, GAMGI_CHEM_DY_RADIUS,
GAMGI_CHEM_DY_R, GAMGI_CHEM_DY_G, GAMGI_CHEM_DY_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_HO, GAMGI_CHEM_HO_MASS, GAMGI_CHEM_HO_RADIUS,
GAMGI_CHEM_HO_R, GAMGI_CHEM_HO_G, GAMGI_CHEM_HO_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_ER, GAMGI_CHEM_ER_MASS, GAMGI_CHEM_ER_RADIUS,
GAMGI_CHEM_ER_R, GAMGI_CHEM_ER_G, GAMGI_CHEM_ER_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TM, GAMGI_CHEM_TM_MASS, GAMGI_CHEM_TM_RADIUS,
GAMGI_CHEM_TM_R, GAMGI_CHEM_TM_G, GAMGI_CHEM_TM_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_YB, GAMGI_CHEM_YB_MASS, GAMGI_CHEM_YB_RADIUS,
GAMGI_CHEM_YB_R, GAMGI_CHEM_YB_G, GAMGI_CHEM_YB_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_LU, GAMGI_CHEM_LU_MASS, GAMGI_CHEM_LU_RADIUS,
GAMGI_CHEM_LU_R, GAMGI_CHEM_LU_G, GAMGI_CHEM_LU_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_HF, GAMGI_CHEM_HF_MASS, GAMGI_CHEM_HF_RADIUS,
GAMGI_CHEM_HF_R, GAMGI_CHEM_HF_G, GAMGI_CHEM_HF_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TA, GAMGI_CHEM_TA_MASS, GAMGI_CHEM_TA_RADIUS,
GAMGI_CHEM_TA_R, GAMGI_CHEM_TA_G, GAMGI_CHEM_TA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_W, GAMGI_CHEM_W_MASS, GAMGI_CHEM_W_RADIUS,
GAMGI_CHEM_W_R, GAMGI_CHEM_W_G, GAMGI_CHEM_W_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RE, GAMGI_CHEM_RE_MASS, GAMGI_CHEM_RE_RADIUS,
GAMGI_CHEM_RE_R, GAMGI_CHEM_RE_G, GAMGI_CHEM_RE_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_OS, GAMGI_CHEM_OS_MASS, GAMGI_CHEM_OS_RADIUS,
GAMGI_CHEM_OS_R, GAMGI_CHEM_OS_G, GAMGI_CHEM_OS_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_IR, GAMGI_CHEM_IR_MASS, GAMGI_CHEM_IR_RADIUS,
GAMGI_CHEM_IR_R, GAMGI_CHEM_IR_G, GAMGI_CHEM_IR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PT, GAMGI_CHEM_PT_MASS, GAMGI_CHEM_PT_RADIUS,
GAMGI_CHEM_PT_R, GAMGI_CHEM_PT_G, GAMGI_CHEM_PT_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AU, GAMGI_CHEM_AU_MASS, GAMGI_CHEM_AU_RADIUS,
GAMGI_CHEM_AU_R, GAMGI_CHEM_AU_G, GAMGI_CHEM_AU_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_HG, GAMGI_CHEM_HG_MASS, GAMGI_CHEM_HG_RADIUS,
GAMGI_CHEM_HG_R, GAMGI_CHEM_HG_G, GAMGI_CHEM_HG_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TL, GAMGI_CHEM_TL_MASS, GAMGI_CHEM_TL_RADIUS,
GAMGI_CHEM_TL_R, GAMGI_CHEM_TL_G, GAMGI_CHEM_TL_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PB, GAMGI_CHEM_PB_MASS, GAMGI_CHEM_PB_RADIUS,
GAMGI_CHEM_PB_R, GAMGI_CHEM_PB_G, GAMGI_CHEM_PB_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_BI, GAMGI_CHEM_BI_MASS, GAMGI_CHEM_BI_RADIUS,
GAMGI_CHEM_BI_R, GAMGI_CHEM_BI_G, GAMGI_CHEM_BI_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PO, GAMGI_CHEM_PO_MASS, GAMGI_CHEM_PO_RADIUS,
GAMGI_CHEM_PO_R, GAMGI_CHEM_PO_G, GAMGI_CHEM_PO_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AT, GAMGI_CHEM_AT_MASS, GAMGI_CHEM_AT_RADIUS,
GAMGI_CHEM_AT_R, GAMGI_CHEM_AT_G, GAMGI_CHEM_AT_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RN, GAMGI_CHEM_RN_MASS, GAMGI_CHEM_RN_RADIUS,
GAMGI_CHEM_RN_R, GAMGI_CHEM_RN_G, GAMGI_CHEM_RN_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_FR, GAMGI_CHEM_FR_MASS, GAMGI_CHEM_FR_RADIUS,
GAMGI_CHEM_FR_R, GAMGI_CHEM_FR_G, GAMGI_CHEM_FR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RA, GAMGI_CHEM_RA_MASS, GAMGI_CHEM_RA_RADIUS,
GAMGI_CHEM_RA_R, GAMGI_CHEM_RA_G, GAMGI_CHEM_RA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AC, GAMGI_CHEM_AC_MASS, GAMGI_CHEM_AC_RADIUS,
GAMGI_CHEM_AC_R, GAMGI_CHEM_AC_G, GAMGI_CHEM_AC_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_TH, GAMGI_CHEM_TH_MASS, GAMGI_CHEM_TH_RADIUS,
GAMGI_CHEM_TH_R, GAMGI_CHEM_TH_G, GAMGI_CHEM_TH_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PA, GAMGI_CHEM_PA_MASS, GAMGI_CHEM_PA_RADIUS,
GAMGI_CHEM_PA_R, GAMGI_CHEM_PA_G, GAMGI_CHEM_PA_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_U, GAMGI_CHEM_U_MASS, GAMGI_CHEM_U_RADIUS,
GAMGI_CHEM_U_R, GAMGI_CHEM_U_G, GAMGI_CHEM_U_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_NP, GAMGI_CHEM_NP_MASS, GAMGI_CHEM_NP_RADIUS,
GAMGI_CHEM_NP_R, GAMGI_CHEM_NP_G, GAMGI_CHEM_NP_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_PU, GAMGI_CHEM_PU_MASS, GAMGI_CHEM_PU_RADIUS,
GAMGI_CHEM_PU_R, GAMGI_CHEM_PU_G, GAMGI_CHEM_PU_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_AM, GAMGI_CHEM_AM_MASS, GAMGI_CHEM_AM_RADIUS,
GAMGI_CHEM_AM_R, GAMGI_CHEM_AM_G, GAMGI_CHEM_AM_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CM, GAMGI_CHEM_CM_MASS, GAMGI_CHEM_CM_RADIUS,
GAMGI_CHEM_CM_R, GAMGI_CHEM_CM_G, GAMGI_CHEM_CM_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_BK, GAMGI_CHEM_BK_MASS, GAMGI_CHEM_BK_RADIUS,
GAMGI_CHEM_BK_R, GAMGI_CHEM_BK_G, GAMGI_CHEM_BK_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_CF, GAMGI_CHEM_CF_MASS, GAMGI_CHEM_CF_RADIUS,
GAMGI_CHEM_CF_R, GAMGI_CHEM_CF_G, GAMGI_CHEM_CF_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_ES, GAMGI_CHEM_ES_MASS, GAMGI_CHEM_ES_RADIUS,
GAMGI_CHEM_ES_R, GAMGI_CHEM_ES_G, GAMGI_CHEM_ES_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_FM, GAMGI_CHEM_FM_MASS, GAMGI_CHEM_FM_RADIUS,
GAMGI_CHEM_FM_R, GAMGI_CHEM_FM_G, GAMGI_CHEM_FM_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_MD, GAMGI_CHEM_MD_MASS, GAMGI_CHEM_MD_RADIUS,
GAMGI_CHEM_MD_R, GAMGI_CHEM_MD_G, GAMGI_CHEM_MD_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_NO, GAMGI_CHEM_NO_MASS, GAMGI_CHEM_NO_RADIUS,
GAMGI_CHEM_NO_R, GAMGI_CHEM_NO_G, GAMGI_CHEM_NO_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_LR, GAMGI_CHEM_LR_MASS, GAMGI_CHEM_LR_RADIUS,
GAMGI_CHEM_LR_R, GAMGI_CHEM_LR_G, GAMGI_CHEM_LR_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RF, GAMGI_CHEM_RF_MASS, GAMGI_CHEM_RF_RADIUS,
GAMGI_CHEM_RF_R, GAMGI_CHEM_RF_G, GAMGI_CHEM_RF_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_DB, GAMGI_CHEM_DB_MASS, GAMGI_CHEM_DB_RADIUS,
GAMGI_CHEM_DB_R, GAMGI_CHEM_DB_G, GAMGI_CHEM_DB_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_SG, GAMGI_CHEM_SG_MASS, GAMGI_CHEM_SG_RADIUS,
GAMGI_CHEM_SG_R, GAMGI_CHEM_SG_G, GAMGI_CHEM_SG_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_BH, GAMGI_CHEM_BH_MASS, GAMGI_CHEM_BH_RADIUS,
GAMGI_CHEM_BH_R, GAMGI_CHEM_BH_G, GAMGI_CHEM_BH_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_HS, GAMGI_CHEM_HS_MASS, GAMGI_CHEM_HS_RADIUS,
GAMGI_CHEM_HS_R, GAMGI_CHEM_HS_G, GAMGI_CHEM_HS_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_MT, GAMGI_CHEM_MT_MASS, GAMGI_CHEM_MT_RADIUS,
GAMGI_CHEM_MT_R, GAMGI_CHEM_MT_G, GAMGI_CHEM_MT_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_DS, GAMGI_CHEM_DS_MASS, GAMGI_CHEM_DS_RADIUS,
GAMGI_CHEM_DS_R, GAMGI_CHEM_DS_G, GAMGI_CHEM_DS_B);

gamgi_expat_export_atom (window, fp, depth_parent, depth_last, error,
GAMGI_CHEM_RG, GAMGI_CHEM_RG_MASS, GAMGI_CHEM_RG_RADIUS,
GAMGI_CHEM_RG_R, GAMGI_CHEM_RG_G, GAMGI_CHEM_RG_B);
}

gamgi_bool gamgi_chem_atom_unlink (gamgi_atom *atom)
{
gamgi_object *parent;
gamgi_plane *plane;
gamgi_direction *direction;
gamgi_atom *atom1, *atom2, *atom3;

parent = atom->object.object;

/*******************************************************
 * This atom is about to be removed or linked to a     *
 * different object, so if it is currently defining    *
 * a plane or direction, remove the plane or direction *
 *******************************************************/

if (parent->class == GAMGI_ENGINE_PLANE)
  {
  plane = GAMGI_CAST_PLANE parent;
  if (plane->reference == GAMGI_ENGINE_ATOM &&
  gamgi_phys_plane_reference (plane, &atom1, &atom2, &atom3) == TRUE &&
  (atom == atom1 || atom == atom2 || atom == atom3))
    {
    /*************************************
     * unlink atom before removing plane *
     *************************************/

    gamgi_engine_unlink_atom (atom);
    
    /***********************************************************
     * link plane child to plane parent, before removing plane *
     ***********************************************************/

    parent = plane->object.object;
    gamgi_engine_reparent_direction (plane->direction_start, parent);
    gamgi_engine_reparent_atom (plane->atom_start, parent);
    gamgi_engine_reparent_orbital (plane->orbital_start, parent);
    gamgi_engine_reparent_text (plane->text_start, parent);

    gamgi_engine_unlink_plane (plane);
    gamgi_engine_remove_plane (plane);
    return TRUE;
    }

  }

if (parent->class == GAMGI_ENGINE_DIRECTION)
  {
  direction = GAMGI_CAST_DIRECTION parent;
  if (direction->reference == GAMGI_ENGINE_ATOM &&
  gamgi_phys_direction_reference (direction, &atom1, &atom2) == TRUE &&
  (atom == atom1 || atom == atom2))
    {
    /*****************************************
     * unlink atom before removing direction *
     *****************************************/

    gamgi_engine_unlink_atom (atom);

    /***********************************************************************
     * link direction child to direction parent, before removing direction *
     ***********************************************************************/

    parent = direction->object.object;
    gamgi_engine_reparent_atom (direction->atom_start, parent);
    gamgi_engine_reparent_orbital (direction->orbital_start, parent);
    gamgi_engine_reparent_text (direction->text_start, parent);

    gamgi_engine_unlink_direction (direction);
    gamgi_engine_remove_direction (direction);
    return TRUE;
    }

  }

gamgi_engine_unlink_atom (atom);
return FALSE;
}
